// Copyright 2023 schukai GmbH
// SPDX-License-Identifier: AGPL-3.0

package document

import (
	"errors"
	"os"
	"path"
	"path/filepath"
	"sort"
)

type SourceFileMap map[string]*SourceFile

func (m SourceFileMap) get(key string) *SourceFile {
	return m[key]
}

func (m SourceFileMap) findByRelativePath(rel string) *SourceFile {
	for _, v := range m {
		if v.relSourcePath == rel {
			return v
			break
		}
	}
	return nil
}

func (m SourceFileMap) findByHash(hash string) *SourceFile {
	for _, v := range m {
		if v.hash == hash {
			return v
			break
		}
	}
	return nil
}

func buildFileMap(files []*SourceFile) (SourceFileMap, []string) {

	sort.Slice(files, func(i, j int) bool {
		if files[i].relSourcePath == files[j].relSourcePath {
			if files[i].level < files[j].level {
				return true
			}

			return files[i].level < files[j].level
		}

		return files[i].relSourcePath < files[j].relSourcePath
	})

	keys := make([]string, 0, len(files))
	mapFiles := make(map[string]*SourceFile)
	for _, file := range files {
		keys = append(keys, file.absSourcePath)
		mapFiles[file.absSourcePath] = file
	}

	return mapFiles, keys

}

func getFiles(tmpl GetComponentsTemplatesInterface, sourcePath string) ([]*SourceFile, error) {

	def := []*SourceFile{}

	err := filepath.Walk(sourcePath,
		func(current string, info os.FileInfo, err error) error {
			if err != nil {
				return err
			}

			if info.IsDir() {
				return nil
			}

			ext := filepath.Ext(current)
			if ext != ".md" {
				return nil
			}

			cp := path.Clean(current)

			c, err := os.ReadFile(cp)
			if err != nil {
				return err
			}

			err, p := evaluateDocumentContent(tmpl, c)

			if err != nil {
				return errors.New("Error while processing files: " + current + " (" + err.Error() + ")")
			}

			def = append(def, NewDefinition(sourcePath, current, info, p))
			return nil
		})

	if err != nil {
		return def, err
	}

	return def, err

}
