// Copyright 2023 schukai GmbH
// SPDX-License-Identifier: AGPL-3.0

package document

import (
	"bytes"
	"encoding/json"
	"errors"
	"html/template"
	"os"
	"path"
	"time"
)

type ThunderClient struct {
	Id          string    `json:"_id"`
	ColId       string    `json:"colId"`
	ContainerId string    `json:"containerId"`
	Name        string    `json:"name"`
	Url         string    `json:"url"`
	Method      string    `json:"method"`
	SortNum     int       `json:"sortNum"`
	Created     time.Time `json:"created"`
	Modified    time.Time `json:"modified"`
	Headers     []struct {
		Name  string `json:"name"`
		Value string `json:"value"`
	} `json:"headers"`
	Params []struct {
		Name   string `json:"name"`
		Value  string `json:"value"`
		isPath bool   `json:"isPath"`
	} `json:"params"`
	Body struct {
		Type string        `json:"type"`
		Raw  string        `json:"raw"`
		Form []interface{} `json:"form"`
	} `json:"body"`
	Tests []struct {
		Type   string `json:"type"`
		Custom string `json:"custom"`
		Action string `json:"action"`
		Value  string `json:"value"`
	} `json:"tests"`
	Docs string `json:"docs"`
}

type ThunderCollection struct {
	Id      string    `json:"_id"`
	ColName string    `json:"colName"`
	Created time.Time `json:"created"`
	SortNum int       `json:"sortNum"`
	Folders []struct {
		Id          string    `json:"_id"`
		Name        string    `json:"name"`
		ContainerId string    `json:"containerId"`
		Created     time.Time `json:"created"`
		SortNum     int       `json:"sortNum"`
	} `json:"folders"`
	Settings struct {
		Headers []interface{} `json:"headers"`
		Auth    struct {
			Type         string `json:"type"`
			Bearer       string `json:"bearer"`
			BearerPrefix string `json:"bearerPrefix"`
		} `json:"auth"`
		Tests []interface{} `json:"tests"`
		Docs  string        `json:"docs"`
	} `json:"settings"`
}

func embedThunderClientAPI(env GetComponentsTemplatesInterface, defPath, id string) string {

	thunderStruct := readThunderStructs(defPath)
	s, done := renderThunderAPI(env, thunderStruct, id)
	if done {
		return "\r\n<div>" + s + "\r\n</div>\r\n"
	}

	return ""
}

func getApiHtmlTemplate() string {
	return ""
	//environment.State.GetHTMLHTMLTemplatePath(definition.Document.HTML.HTMLTemplate)
	//
	//templ, err := template.New("api").Funcs(getFuncMap()).Parse(apiTemplate)
	//checkError(err)
	//
	//var buf bytes.Buffer
	//err = templ.Execute(&buf, ThunderClient{})
	//checkError(err)
	//
	//fmt.Println(buf.String())
}

func renderThunderAPI(env GetComponentsTemplatesInterface, thunder ThunderStruct, id string) (string, bool) {

	tmplMap := env.GetComponentsTemplates()
	tmpl, ok := tmplMap["#doc-api-request"]
	if !ok {
		tmpl = ""
	}

	for _, client := range thunder.ThunderClient {
		if client.Id == id {

			templ, err := template.New("api-template").Funcs(getFuncMap(&Dummy{})).Parse(tmpl)
			checkError(err)

			var buf bytes.Buffer
			err = templ.Execute(&buf, client)
			checkError(err)

			return buf.String(), true

		}
	}

	for _, collection := range thunder.ThunderCollection {
		if collection.Id != id {
			continue
		}

		result := ""
		found := false

		for _, tc := range thunder.ThunderClient {
			if tc.ColId == collection.Id {
				r, b := renderThunderAPI(env, thunder, tc.Id)
				if b {
					found = true
					result += r
				}
			}
		}

		if found {
			return result, true
		}

	}

	return "", false
}

type ThunderStruct struct {
	ThunderClient     []ThunderClient
	ThunderCollection []ThunderCollection
}

func readThunderStructs(defPath string) ThunderStruct {

	if defPath == "" {
		checkError(errors.New("path is empty"))
	}

	stat, err := os.Stat(defPath)
	if errors.Is(err, os.ErrNotExist) {
		checkError(err)
	}

	if !stat.IsDir() {
		checkError(errors.New("path is not a directory"))
	}

	j, err := os.ReadFile(path.Join(defPath, "thunderclient.json"))
	checkError(err)

	var thunderClient []ThunderClient

	err = json.Unmarshal(j, &thunderClient)
	checkError(err)

	c, err := os.ReadFile(path.Join(defPath, "thunderCollection.json"))
	checkError(err)

	var thunderCollection []ThunderCollection

	err = json.Unmarshal(c, &thunderCollection)
	checkError(err)

	return ThunderStruct{
		ThunderClient:     thunderClient,
		ThunderCollection: thunderCollection,
	}

}
