package logging

import (
	"github.com/natefinch/lumberjack"
	"go.uber.org/zap"
	"go.uber.org/zap/zapcore"
	"os"
)

var logger *zap.Logger
var sugaredLogger *zap.SugaredLogger

//// Logger for GORM
//func GetDBZornLogger() zapgorm2.Logger {
//	return zapgorm2.New(logger)
//}

func safeFileWriteSyncer(filename string) zapcore.WriteSyncer {

	lumberJackLogger := &lumberjack.Logger{
		Filename:   filename,
		MaxSize:    10,
		MaxBackups: 5,
		MaxAge:     30,
		Compress:   false,
	}
	return zapcore.AddSync(lumberJackLogger)

}

func InitConsoleLogger(level string) {

	// Create a WriteSyncer that handles errors
	syncer := zapcore.AddSync(os.Stdout)

	// Create a new Zap logger instance with a SugaredLogger interface
	logger = zap.New(zapcore.NewCore(
		zapcore.NewConsoleEncoder(zap.NewProductionEncoderConfig()),
		syncer,
		getLogLevelFromString(level),
	))

	// Create a new SugaredLogger instance
	sugaredLogger = logger.Sugar()
}

func getLogLevelFromString(level string) zapcore.Level {
	switch level {
	case "debug":
		return zapcore.DebugLevel
	case "info":
		return zapcore.InfoLevel
	case "warn":
		return zapcore.WarnLevel
	case "error":
		return zapcore.ErrorLevel
	case "panic":
		return zapcore.PanicLevel
	case "fatal":
		return zapcore.FatalLevel
	default:
		return zapcore.ErrorLevel
	}
}

func InitFileLogger(level string, filename string) {

	// Create a WriteSyncer that handles errors
	syncer := safeFileWriteSyncer(filename)

	// Create a new Zap logger instance with a SugaredLogger interface
	logger = zap.New(zapcore.NewCore(
		zapcore.NewJSONEncoder(zap.NewProductionEncoderConfig()),
		syncer,
		getLogLevelFromString(level),
	))

	// Create a new SugaredLogger instance
	sugaredLogger = logger.Sugar()

}

func LogInfo(template string, args ...interface{}) {
	if sugaredLogger == nil {
		InitConsoleLogger("debug")
	}

	if args == nil {
		sugaredLogger.Info(template)
	} else {
		sugaredLogger.Infof(template, args)
	}

}

func LogDebug(template string, args ...interface{}) {
	if sugaredLogger == nil {
		InitConsoleLogger("debug")
	}

	if args == nil {
		sugaredLogger.Debug(template)
	} else {
		sugaredLogger.Debugf(template, args)
	}

}

func LogWarn(template string, args ...interface{}) {
	if sugaredLogger == nil {
		InitConsoleLogger("debug")
	}

	if args == nil {
		sugaredLogger.Warn(template)
	} else {
		sugaredLogger.Warnf(template, args)
	}
}

func LogError(template string, args ...interface{}) {
	if sugaredLogger == nil {
		InitConsoleLogger("debug")
	}

	if args == nil {
		sugaredLogger.Error(template)
	} else {
		sugaredLogger.Errorf(template, args...)
	}

}
